@@ -1,4 +1,3 @@
-from __future__ import annotations
 from typing import Optional, Tuple, Union
 
 OF = Optional[float]
@@ -6,4 +5,4 @@ OI = Optional[int]
 OS = Optional[str]
 TBounds = Union[str, Tuple[OF, OF]]
 TColors = Tuple[OS, OS]
-TLevels = Union[Tuple[OF, OF, OI], float]
+TLevels = Union[Tuple[OF, OF, OI], float, int]
